@@ -3,16 +3,11 @@ import unittest
 
 import numpy as np
 
-from openmdao.api import Problem, NonlinearBlockGS, Group, IndepVarComp, ExecComp, ScipyKrylov,  \
-    IndepVarComp, ScipyOptimizeDriver
-from openmdao.utils.assert_utils import assert_near_equal
+import openmdao.api as om
 from openmdao.utils.mpi import MPI
-
 from openmdao.test_suite.components.sellar import SellarDerivatives, SellarDis1withDerivatives, \
      SellarDis2withDerivatives
 
-from openmdao.core.tests.test_distribcomp import DistribInputDistribOutputComp
-
 try:
     from openmdao.vectors.petsc_vector import PETScVector
 except ImportError:
@@ -23,10 +18,10 @@ class TestDesVarsResponses(unittest.TestCase):
 
     def test_api_on_model(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         prob.model.add_design_var('x', lower=-100, upper=100)
         prob.model.add_design_var('z', lower=-100, upper=100)
@@ -46,10 +41,10 @@ class TestDesVarsResponses(unittest.TestCase):
 
     def test_api_response_on_model(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         prob.model.add_design_var('x', lower=-100, upper=100)
         prob.model.add_design_var('z', lower=-100, upper=100)
@@ -71,10 +66,10 @@ class TestDesVarsResponses(unittest.TestCase):
 
     def test_api_list_on_model(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         prob.model.add_design_var('x', lower=-100, upper=100)
         prob.model.add_design_var('z', lower=[-100, -20], upper=[100, 20])
@@ -94,10 +89,10 @@ class TestDesVarsResponses(unittest.TestCase):
 
     def test_api_array_on_model(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         prob.model.add_design_var('x', lower=-100, upper=100)
         prob.model.add_design_var('z',
@@ -119,10 +114,10 @@ class TestDesVarsResponses(unittest.TestCase):
 
     def test_api_iter_on_model(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         prob.model.add_design_var('x', lower=-100, upper=100)
         prob.model.add_design_var('z', lower=range(-101, -99),
@@ -145,28 +140,28 @@ class TestDesVarsResponses(unittest.TestCase):
 
     def test_api_on_subsystems(self):
 
-        prob = Problem()
+        prob = om.Problem()
         model = prob.model
 
-        model.add_subsystem('px', IndepVarComp('x', 1.0))
-        model.add_subsystem('pz', IndepVarComp('z', np.array([5.0, 2.0])))
+        model.add_subsystem('px', om.IndepVarComp('x', 1.0))
+        model.add_subsystem('pz', om.IndepVarComp('z', np.array([5.0, 2.0])))
 
         model.add_subsystem('d1', SellarDis1withDerivatives())
         model.add_subsystem('d2', SellarDis2withDerivatives())
 
-        model.add_subsystem('obj_cmp', ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
-                                               z=np.array([0.0, 0.0]), x=0.0))
+        model.add_subsystem('obj_cmp', om.ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
+                                                   z=np.array([0.0, 0.0]), x=0.0))
 
-        model.add_subsystem('con_cmp1', ExecComp('con1 = 3.16 - y1'))
-        model.add_subsystem('con_cmp2', ExecComp('con2 = y2 - 24.0'))
+        model.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1'))
+        model.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'))
 
         model.connect('px.x', ['d1.x', 'obj_cmp.x'])
         model.connect('pz.z', ['d1.z', 'd2.z', 'obj_cmp.z'])
         model.connect('d1.y1', ['d2.y1', 'obj_cmp.y1', 'con_cmp1.y1'])
         model.connect('d2.y2', ['d1.y2', 'obj_cmp.y2', 'con_cmp2.y2'])
 
-        model.nonlinear_solver = NonlinearBlockGS()
-        model.linear_solver = ScipyKrylov()
+        model.nonlinear_solver = om.NonlinearBlockGS()
+        model.linear_solver = om.ScipyKrylov()
 
         px = prob.model.px
         px.add_design_var('x', lower=-100, upper=100)
@@ -193,15 +188,75 @@ class TestDesVarsResponses(unittest.TestCase):
         self.assertEqual(set(obj.keys()), {'obj_cmp.obj',})
         self.assertEqual(set(constraints.keys()), {'con_cmp1.con1', 'con_cmp2.con2'})
 
+    def test_units_checking(self):
+        p = om.Problem()
+        model = p.model
+        G1 = model.add_subsystem('G1', om.Group())
+        G1.add_subsystem('C1', om.ExecComp('y = 3.*x', x={'units': 'm'}), promotes=['x'])
+        G1.add_subsystem('C2', om.ExecComp('yy = 4.*xx', xx={'units': 'm'}), promotes=['xx'])
+
+        # Constraints
+
+        with self.assertRaises(ValueError) as cm:
+            G1.add_constraint('y', units='junk')
+
+        msg = "'G1' <class Group>: The units 'junk' are invalid for response 'y'."
+        self.assertEqual(cm.exception.args[0], msg)
+
+        with self.assertRaises(TypeError) as cm:
+            G1.add_constraint('y', units=3)
+
+        msg = "'G1' <class Group>: The units argument should be a str or None for response 'y'."
+        self.assertEqual(cm.exception.args[0], msg)
+
+        G1.add_constraint('y', units='ft*ft/ft')
+        self.assertEqual(G1._static_responses['y']['units'], 'ft')
+
+        # Objectives
+
+        with self.assertRaises(ValueError) as cm:
+            G1.add_objective('yy', units='junk')
+
+        msg = "'G1' <class Group>: The units 'junk' are invalid for response 'yy'."
+        self.assertEqual(cm.exception.args[0], msg)
+
+        with self.assertRaises(TypeError) as cm:
+            G1.add_objective('yy', units=3)
+
+        msg = "'G1' <class Group>: The units argument should be a str or None for response 'yy'."
+        self.assertEqual(cm.exception.args[0], msg)
+
+        # Simplification
+        G1.add_objective('yy', units='ft*ft/ft')
+        self.assertEqual(G1._static_responses['yy']['units'], 'ft')
+
+        # Design Variables
+
+        with self.assertRaises(ValueError) as cm:
+            G1.add_design_var('x', units='junk')
+
+        msg = "'G1' <class Group>: The units 'junk' are invalid for design_var 'x'."
+        self.assertEqual(cm.exception.args[0], msg)
+
+        with self.assertRaises(TypeError) as cm:
+            G1.add_design_var('x', units=3)
+
+        msg = "'G1' <class Group>: The units argument should be a str or None for design_var 'x'."
+        self.assertEqual(cm.exception.args[0], msg)
+
+        # Simplification
+        G1.add_design_var('x', units='ft*ft/ft')
+        self.assertEqual(G1._static_design_vars['x']['units'], 'ft')
+
 
 class TestDesvarOnModel(unittest.TestCase):
 
     def test_design_var_not_exist(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         prob.model.add_design_var('junk')
 
@@ -212,10 +267,10 @@ class TestDesvarOnModel(unittest.TestCase):
 
     def test_desvar_affine_and_scaleradder(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         with self.assertRaises(ValueError) as context:
             prob.model.add_design_var('x', lower=-100, upper=100, ref=1.0,
@@ -251,10 +306,10 @@ class TestDesvarOnModel(unittest.TestCase):
 
     def test_desvar_affine_mapping(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         prob.model.add_design_var('x', lower=-100, upper=100, ref0=-100.0,
                                   ref=100)
@@ -279,10 +334,10 @@ class TestDesvarOnModel(unittest.TestCase):
 
         # make sure no overflow when there is no specified upper/lower bound and significatn scaling
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         prob.model.add_design_var('x', scaler=1e6)
         prob.model.add_objective('obj', scaler=1e6)
@@ -305,10 +360,10 @@ class TestDesvarOnModel(unittest.TestCase):
 
     def test_desvar_invalid_name(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         with self.assertRaises(TypeError) as context:
             prob.model.add_design_var(42, lower=-100, upper=100, ref0=-100.0,
@@ -319,10 +374,10 @@ class TestDesvarOnModel(unittest.TestCase):
 
     def test_desvar_invalid_bounds(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         with self.assertRaises(TypeError) as context:
             prob.model.add_design_var('x', lower='foo', upper=[0, 100],
@@ -341,10 +396,10 @@ class TestConstraintOnModel(unittest.TestCase):
 
     def test_constraint_not_exist(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         prob.model.add_constraint('junk')
 
@@ -355,10 +410,10 @@ class TestConstraintOnModel(unittest.TestCase):
 
     def test_constraint_affine_and_scaleradder(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         with self.assertRaises(ValueError) as context:
             prob.model.add_constraint('con1', lower=-100, upper=100, ref=1.0,
@@ -394,10 +449,10 @@ class TestConstraintOnModel(unittest.TestCase):
 
     def test_constraint_affine_mapping(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         prob.model.add_design_var('x', lower=-100, upper=100)
         prob.model.add_design_var('z', lower=-100, upper=100)
@@ -422,10 +477,10 @@ class TestConstraintOnModel(unittest.TestCase):
 
     def test_constraint_invalid_name(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         with self.assertRaises(TypeError) as context:
             prob.model.add_design_var(42, lower=-100, upper=100, ref0=-100.0,
@@ -436,10 +491,10 @@ class TestConstraintOnModel(unittest.TestCase):
 
     def test_constraint_invalid_bounds(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         with self.assertRaises(TypeError) as context:
             prob.model.add_design_var('x', lower='foo', upper=[0, 100],
@@ -456,10 +511,10 @@ class TestConstraintOnModel(unittest.TestCase):
 
     def test_constraint_invalid_name(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         with self.assertRaises(TypeError) as context:
             prob.model.add_constraint(42, lower=-100, upper=100, ref0=-100.0,
@@ -470,9 +525,9 @@ class TestConstraintOnModel(unittest.TestCase):
 
     def test_constraint_invalid_lower(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
-        prob.driver = ScipyOptimizeDriver()
+        prob.driver = om.ScipyOptimizeDriver()
         prob.driver.options['optimizer'] = 'SLSQP'
 
         with self.assertRaises(TypeError) as context:
@@ -495,9 +550,9 @@ class TestConstraintOnModel(unittest.TestCase):
 
     def test_constraint_invalid_upper(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
-        prob.driver = ScipyOptimizeDriver()
+        prob.driver = om.ScipyOptimizeDriver()
         prob.driver.options['optimizer'] = 'SLSQP'
 
         with self.assertRaises(TypeError) as context:
@@ -519,9 +574,9 @@ class TestConstraintOnModel(unittest.TestCase):
         self.assertEqual(str(context2.exception), msg2)
 
     def test_constraint_invalid_equals(self):
-        prob = Problem()
+        prob = om.Problem()
 
-        prob.driver = ScipyOptimizeDriver()
+        prob.driver = om.ScipyOptimizeDriver()
         prob.driver.options['optimizer'] = 'SLSQP'
 
         with self.assertRaises(TypeError) as context:
@@ -542,10 +597,10 @@ class TestConstraintOnModel(unittest.TestCase):
 
     def test_constraint_invalid_indices(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         with self.assertRaises(ValueError) as context:
             prob.model.add_constraint('con1', lower=0.0, upper=5.0,
@@ -573,10 +628,10 @@ class TestConstraintOnModel(unittest.TestCase):
                                           indices=range(2))
 
     def test_error_eq_ineq_con(self):
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         with self.assertRaises(ValueError) as context:
             prob.model.add_constraint('con1', lower=0.0, upper=5.0, equals=3.0,
@@ -593,11 +648,11 @@ class TestAddConstraintMPI(unittest.TestCase):
 
     def test_add_bad_con(self):
         # From a bug, this message didn't work in mpi.
-        prob = Problem()
+        prob = om.Problem()
         model = prob.model
 
         sub = model.add_subsystem('sub', SellarDerivatives())
-        sub.nonlinear_solver = NonlinearBlockGS()
+        sub.nonlinear_solver = om.NonlinearBlockGS()
 
         sub.add_constraint('d1.junk', equals=0.0, cache_linear_solution=True)
 
@@ -612,10 +667,10 @@ class TestObjectiveOnModel(unittest.TestCase):
 
     def test_obective_not_exist(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         prob.model.add_objective('junk')
 
@@ -627,10 +682,10 @@ class TestObjectiveOnModel(unittest.TestCase):
 
     def test_objective_affine_and_scaleradder(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         with self.assertRaises(TypeError) as context:
             prob.model.add_objective('con1', lower=-100, upper=100, ref=1.0,
@@ -669,10 +724,10 @@ class TestObjectiveOnModel(unittest.TestCase):
 
     def test_objective_affine_mapping(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         prob.model.add_design_var('x', lower=-100, upper=100)
         prob.model.add_design_var('z', lower=-100, upper=100)
@@ -695,10 +750,10 @@ class TestObjectiveOnModel(unittest.TestCase):
 
     def test_desvar_size_err(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         for name in ['lower', 'upper', 'adder', 'scaler', 'ref', 'ref0']:
             args = {name: -np.ones(2)*100}
@@ -709,10 +764,10 @@ class TestObjectiveOnModel(unittest.TestCase):
 
     def test_constraint_size_err(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         for name in ['lower', 'upper', 'equals', 'adder', 'scaler', 'ref', 'ref0']:
             args = {name: -np.ones(2)*100}
@@ -723,10 +778,10 @@ class TestObjectiveOnModel(unittest.TestCase):
 
     def test_objective_size_err(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         for name in ['adder', 'scaler', 'ref', 'ref0']:
             args = {name: -np.ones(2)*100}
@@ -737,10 +792,10 @@ class TestObjectiveOnModel(unittest.TestCase):
 
     def test_objective_invalid_name(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         with self.assertRaises(TypeError) as context:
             prob.model.add_objective(42, ref0=-100.0, ref=100)
@@ -750,10 +805,10 @@ class TestObjectiveOnModel(unittest.TestCase):
 
     def test_objective_invalid_index(self):
 
-        prob = Problem()
+        prob = om.Problem()
 
         prob.model = SellarDerivatives()
-        prob.model.nonlinear_solver = NonlinearBlockGS()
+        prob.model.nonlinear_solver = om.NonlinearBlockGS()
 
         with self.assertRaises(TypeError) as context:
             prob.model.add_objective('obj', index='foo')
