@@ -461,6 +461,9 @@ def insert_output_start_stop_indicators(src):
         '.list_inputs(',
         '.list_outputs(',
         '.list_problem_vars(',
+        '.list_cases(',
+        '.list_model_options(',
+        '.list_solver_options(',
     ]
 
     newlines = []
