@@ -32,7 +32,7 @@ class ParaboloidAE(om.ExplicitComponent):
     The AE in ParaboloidAE stands for AnalysisError."""
 
     def __init__(self):
-        super(ParaboloidAE, self).__init__()
+        super().__init__()
         self.fail_hard = False
 
     def setup(self):
@@ -86,7 +86,7 @@ class ParaboloidAE(om.ExplicitComponent):
         self.grad_iter_count += 1
 
 
-class DummyComp(om.ExecComp):
+class DummyComp(om.ExplicitComponent):
     """
     Evaluates the equation f(x,y) = (x-3)^2 + xy + (y+4)^2 - 3.
     """
@@ -96,7 +96,7 @@ class DummyComp(om.ExecComp):
 
         self.add_output('c', val=0.0)
 
-        self.declare_partials('*', '*')
+        self.declare_partials('*', '*', method='cs')
 
     def compute(self, inputs, outputs):
         """
@@ -1248,6 +1248,7 @@ class TestPyoptSparse(unittest.TestCase):
                 self.add_input('y2', val=0.0)
                 self.add_output('y1', val=0.0)
 
+            def setup_partials(self):
                 self.declare_partials(of='*', wrt='*')
 
             def compute(self, inputs, outputs):
@@ -1271,6 +1272,7 @@ class TestPyoptSparse(unittest.TestCase):
                 self.add_input('y1', val=0.0)
                 self.add_output('y2', val=0.0)
 
+            def setup_partials(self):
                 self.declare_partials(of='*', wrt='*')
 
             def compute(self, inputs, outputs):
@@ -2315,13 +2317,13 @@ class TestPyoptSparseSnoptFeature(unittest.TestCase):
                 self.add_input('x', val=0.)
                 self.add_input('y2', val=1.0)
                 self.add_output('y1', val=1.0)
-
-                self.declare_partials('*', '*')
-
                 self.fail_deriv = [2, 4]
                 self.count_iter = 0
                 self.failed = 0
 
+            def setup_partials(self):
+                self.declare_partials('*', '*')
+
             def compute(self, inputs, outputs):
 
                 z1 = inputs['z'][0]
@@ -2348,6 +2350,7 @@ class TestPyoptSparseSnoptFeature(unittest.TestCase):
                 self.add_input('y1', val=1.0)
                 self.add_output('y2', val=1.0)
 
+            def setup_partials(self):
                 self.declare_partials('*', '*')
 
             def compute(self, inputs, outputs):
@@ -2456,7 +2459,7 @@ class TestPyoptSparseSnoptFeature(unittest.TestCase):
 
 class MatMultCompExact(om.ExplicitComponent):
     def __init__(self, mat, sparse=False, **kwargs):
-        super(MatMultCompExact, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.mat = mat
         self.sparse = sparse
 
@@ -2494,7 +2497,7 @@ class MatMultCompExact(om.ExplicitComponent):
 
 class MyGroup(om.Group):
     def __init__(self, size=5):
-        super(MyGroup, self).__init__()
+        super().__init__()
         self.size = size
 
     def setup(self):
