@@ -4,20 +4,18 @@ Definition of the SqliteCaseReader.
 import sqlite3
 from collections import OrderedDict
 
-from io import StringIO
-
 import sys
 import numpy as np
 
 from openmdao.recorders.base_case_reader import BaseCaseReader
 from openmdao.recorders.case import Case
-
+from openmdao.core.notebook_mode import notebook, tabulate
 from openmdao.core.constants import _DEFAULT_OUT_STREAM
 from openmdao.utils.general_utils import simple_warning
 from openmdao.utils.variable_table import write_source_table
 from openmdao.utils.record_util import check_valid_sqlite3_db, get_source_system
 
-from openmdao.recorders.sqlite_recorder import format_version
+from openmdao.recorders.sqlite_recorder import format_version, META_KEY_SEP
 
 import pickle
 from json import loads as json_loads
@@ -163,7 +161,7 @@ class SqliteCaseReader(BaseCaseReader):
         self._format_version = version = row['format_version']
 
         if version not in range(1, format_version + 1):
-            raise ValueError('SQliteCaseReader encountered an unhandled '
+            raise ValueError('SqliteCaseReader encountered an unhandled '
                              'format version: {0}'.format(self._format_version))
 
         if version >= 11:
@@ -342,10 +340,11 @@ class SqliteCaseReader(BaseCaseReader):
         if self._format_version >= 2 and self._problem_cases.count() > 0:
             sources.extend(self._problem_cases.list_sources())
 
-        if out_stream:
+        if notebook and tabulate is not None:
+            return tabulate([sources], headers=["Source"], tablefmt='html')
+        elif out_stream:
             if out_stream is _DEFAULT_OUT_STREAM:
                 out_stream = sys.stdout
-
             for source in sources:
                 out_stream.write('{}\n'.format(source))
 
@@ -409,14 +408,47 @@ class SqliteCaseReader(BaseCaseReader):
 
         return dct
 
-    def list_model_options(self, run_counter=None, out_stream=_DEFAULT_OUT_STREAM):
+    def systems(self, tree=None, path=None, paths=[]):
         """
-        List of all model options.
+        List pathnames of systems in the system hierarchy.
 
         Parameters
         ----------
-        run_counter : int or None
+        tree : dict
+            Nested dictionary of system information
+        path : str or None
+            Pathname of root system (None for the root model)
+        paths : list
+            List to which pathnames are appended
+
+        Returns
+        -------
+        list
+            List of pathnames of systems
+        """
+        if tree is None:
+            tree = self.problem_metadata['tree']
+
+        path = '.'.join([path, tree['name']]) if path else tree['name']
+        paths.append(path)
+
+        if 'children' in tree:
+            for child in tree['children']:
+                if child['type'] == 'subsystem':
+                    self.systems(child, path, paths)
+
+        return paths
+
+    def list_model_options(self, run_number=0, system=None, out_stream=_DEFAULT_OUT_STREAM):
+        """
+        List model options for the specified run.
+
+        Parameters
+        ----------
+        run_number : int
             Run_driver or run_model iteration to inspect
+        system : str or None
+            Pathname of system (None for all systems)
         out_stream : file-like object
             Where to send human readable output. Default is sys.stdout.
             Set to None to suppress.
@@ -424,34 +456,108 @@ class SqliteCaseReader(BaseCaseReader):
         Returns
         -------
         dict
-            {'root':{key val}}
+            {system: {key: val}}
         """
-        if out_stream:
-            if out_stream is _DEFAULT_OUT_STREAM:
-                out_stream = sys.stdout
+        dct = {}
 
-            dct = {}
+        if not self._system_options:
+            simple_warning("System options not recorded.")
+            return dct
 
-            for i in self._system_options:
-                if '_' in i:
-                    subsys, num = i.rsplit('_', 1)
-                else:
-                    subsys = i
-                    num = 0
+        if out_stream is _DEFAULT_OUT_STREAM:
+            out_stream = sys.stdout
 
-                if (run_counter is not None and run_counter == int(num) and subsys == 'root') or \
-                        (subsys == 'root' and run_counter is None):
+        num_header = None
 
-                    out_stream.write(
-                        'Run Number: {}\n    Subsystem: {}'.format(num, subsys))
+        # need to handle edge case for v11 recording
+        if self._format_version < 12:
+            SEP = '_'
+        else:
+            SEP = META_KEY_SEP
 
-                    for j in self._system_options[i]['component_options']:
-                        option = "{0} : {1}".format(
-                            j, self._system_options[i]['component_options'][j])
-                        out_stream.write('\n        {}\n'.format(option))
+        for key in self._system_options:
+            if key.find(SEP) > 0:
+                name, num = key.rsplit(SEP, 1)
+            else:
+                name = key
+                num = 0
 
-                        dct[subsys] = {}
-                        dct[subsys][j] = self._system_options[i]['component_options'][j]
+            if (system is None or system == name) and (run_number == int(num)):
+
+                if out_stream:
+                    if num_header != num:
+                        out_stream.write(f"Run Number: {num}\n")
+                        num_header = num
+
+                    out_stream.write(f"    Subsystem: {name}\n")
+
+                dct[name] = {}
+
+                comp_options = self._system_options[key]['component_options']
+
+                for opt, val in comp_options.items():
+                    dct[name][opt] = val
+
+                    if out_stream:
+                        out_stream.write(f"        {opt} : {val}\n")
+
+        return dct
+
+    def list_solver_options(self, run_number=0, solver=None, out_stream=_DEFAULT_OUT_STREAM):
+        """
+        List solver options for the specified run.
+
+        Parameters
+        ----------
+        run_number : int
+            Run_driver or run_model iteration to inspect
+        solver : str or None
+            Pathname of solver (None for all solvers)
+        out_stream : file-like object
+            Where to send human readable output. Default is sys.stdout.
+            Set to None to suppress.
+
+        Returns
+        -------
+        dict
+            {solver: {key: val}}
+        """
+        dct = {}
+
+        if not self.solver_metadata:
+            simple_warning("Solver options not recorded.")
+            return dct
+
+        if out_stream is _DEFAULT_OUT_STREAM:
+            out_stream = sys.stdout
+
+        num_header = None
+
+        for key in self.solver_metadata:
+            if key.find(META_KEY_SEP) > 0:
+                name, num = key.rsplit(META_KEY_SEP, 1)
+            else:
+                name = key
+                num = 0
+
+            if (solver is None or solver == name) and (run_number == int(num)):
+
+                if out_stream:
+                    if num_header != num:
+                        out_stream.write(f"Run Number: {num}\n")
+                        num_header = num
+
+                    out_stream.write(f"    Solver: {name}\n")
+
+                dct[name] = {}
+
+                comp_options = self.solver_metadata[key]['solver_options']
+
+                for opt, val in comp_options.items():
+                    dct[name][opt] = val
+
+                    if out_stream:
+                        out_stream.write(f"        {opt} : {val}\n")
 
         return dct
 
@@ -500,7 +606,11 @@ class SqliteCaseReader(BaseCaseReader):
                             (source, type(source).__name__))
 
         if not source:
-            return self._list_cases_recurse_flat(out_stream=out_stream)
+            if notebook and tabulate is not None:
+                cases = self._list_cases_recurse_flat(out_stream=out_stream)
+                return tabulate(cases, headers="keys", tablefmt='html')
+            else:
+                return self._list_cases_recurse_flat(out_stream=out_stream)
 
         elif source == 'problem':
             if self._format_version >= 2:
@@ -521,7 +631,10 @@ class SqliteCaseReader(BaseCaseReader):
                 case_table = None
 
             if case_table is not None:
-                if not recurse:
+                if notebook and tabulate is not None:
+                    cases = [[case] for case in case_table._cases.keys()]
+                    return tabulate(cases, headers=[source], tablefmt='html')
+                elif not recurse:
                     # return list of cases from the source alone
                     return case_table.list_cases(source)
                 elif flat:
@@ -615,7 +728,11 @@ class SqliteCaseReader(BaseCaseReader):
             if out_stream is _DEFAULT_OUT_STREAM:
                 out_stream = sys.stdout
 
-            write_source_table(self.source_cases_table, out_stream)
+            if notebook:
+                nb_format = {key: [val] for key, val in self.source_cases_table.items() if val}
+                return nb_format
+            else:
+                write_source_table(self.source_cases_table, out_stream)
 
         return cases
 
