@@ -19,9 +19,10 @@ def database_cursor(filename):
     con = sqlite3.connect(filename)
     cur = con.cursor()
 
-    yield cur
-
-    con.close()
+    try:
+        yield cur
+    finally:
+        con.close()
 
 
 def get_format_version_abs2meta(db_cur):
