@@ -4,15 +4,19 @@ import inspect
 import json
 import os
 import zlib
-from collections import OrderedDict
-from itertools import chain
 import networkx as nx
 
 import numpy as np
 
+try:
+    from IPython.display import IFrame, display
+except ImportError:
+    IFrame = display = None
+
 from openmdao.components.exec_comp import ExecComp
 from openmdao.components.meta_model_structured_comp import MetaModelStructuredComp
 from openmdao.components.meta_model_unstructured_comp import MetaModelUnStructuredComp
+from openmdao.core.notebook_mode import notebook
 from openmdao.core.explicitcomponent import ExplicitComponent
 from openmdao.core.indepvarcomp import IndepVarComp
 from openmdao.core.parallel_group import ParallelGroup
@@ -94,7 +98,7 @@ def _get_var_dict(system, typ, name, is_parallel):
         name = system._var_abs2prom[typ][name]
         is_discrete = False
 
-    var_dict = OrderedDict()
+    var_dict = {}
 
     var_dict['name'] = name
     var_dict['type'] = typ
@@ -179,7 +183,7 @@ def _serialize_single_option(option):
 def _get_tree_dict(system, component_execution_orders, component_execution_index,
                    is_parallel=False):
     """Get a dictionary representation of the system hierarchy."""
-    tree_dict = OrderedDict()
+    tree_dict = {}
     tree_dict['name'] = system.name
     tree_dict['type'] = 'subsystem'
     tree_dict['class'] = system.__class__.__name__
@@ -534,6 +538,7 @@ def n2(data_source, outfile='n2.html', show_browser=True, embeddable=False,
         'N2TreeNode', \
         'ModelData', \
         'N2Style', \
+        'N2Window', \
         'N2Layout', \
         'N2MatrixCell', \
         'N2Legend', \
@@ -550,10 +555,10 @@ def n2(data_source, outfile='n2.html', show_browser=True, embeddable=False,
     srcs = read_files(src_names, src_dir, 'js')
 
     style_names = \
+        'window', \
         'partition_tree', \
         'icon', \
         'toolbar', \
-        'nodedata', \
         'legend', \
         'awesomplete'
 
@@ -568,14 +573,20 @@ def n2(data_source, outfile='n2.html', show_browser=True, embeddable=False,
     with open(os.path.join(assets_dir, "spinner.png"), "rb") as f:
         waiting_icon = str(base64.b64encode(f.read()).decode("ascii"))
 
+    with open(os.path.join(assets_dir, "toolbar_help.svg"), "r") as f:
+        help_svg = str(f.read())
+
     if title:
         title = "OpenMDAO Model Hierarchy and N2 diagram: %s" % title
     else:
         title = "OpenMDAO Model Hierarchy and N2 diagram"
 
+    src_names = ('N2ErrorHandling',)
+    head_srcs = read_files(src_names, src_dir, 'js')
+
     h = DiagramWriter(filename=os.path.join(vis_dir, "index.html"),
                       title=title,
-                      styles=styles, embeddable=embeddable)
+                      styles=styles, embeddable=embeddable, head_srcs=head_srcs)
 
     if (embeddable):
         h.insert("non-embedded-n2", "embedded-n2")
@@ -584,6 +595,7 @@ def n2(data_source, outfile='n2.html', show_browser=True, embeddable=False,
     h.insert('{{fontello}}', encoded_font)
     h.insert('{{logo_png}}', logo_png)
     h.insert('{{waiting_icon}}', waiting_icon)
+    h.insert('{{help}}', help_svg)
 
     for k, v in lib_dct.items():
         h.insert('{{{}_lib}}'.format(k), write_script(libs[v], indent=_IND))
@@ -594,18 +606,14 @@ def n2(data_source, outfile='n2.html', show_browser=True, embeddable=False,
 
     h.insert('{{model_data}}', write_script(model_data, indent=_IND))
 
-    # Help
-    help_txt = ('Left clicking on a node in the partition tree will navigate to that node. '
-                'Right clicking on a node in the model hierarchy will collapse/expand it. '
-                'A click on any element in the N2 diagram will allow those arrows to persist.')
-    help_diagram_svg_filepath = os.path.join(assets_dir, "toolbar_help.svg")
-    h.add_help(help_txt, help_diagram_svg_filepath,
-               footer="OpenMDAO Model Hierarchy and N2 diagram")
-
     # Write output file
     h.write(outfile)
 
+    # Open in Jupyter Notebook
+    if notebook:
+        display(IFrame(src=outfile, width=1000, height=1000))
+
     # open it up in the browser
-    if show_browser:
+    if show_browser and not notebook:
         from openmdao.utils.webview import webview
         webview(outfile)
