@@ -261,8 +261,29 @@ class N2Toolbar {
                 self._setRootButton(target);
             });
 
+        new N2ToolbarButtonClick('#hide-connections', tooltipBox,
+            "Control connection arrows",
+            function (target) {
+                n2ui.n2Diag.clearArrows();
+                self._setRootButton(target);
+            });
+
+        new N2ToolbarButtonClick('#hide-connections-2', tooltipBox,
+            "Remove all connection arrows",
+            function (target) {
+                n2ui.n2Diag.clearArrows();
+                self._setRootButton(target);
+            });
+
+        new N2ToolbarButtonClick('#show-all-connections', tooltipBox,
+            "Show all connections in view",
+            function (target) {
+                n2ui.n2Diag.showAllArrows();
+                self._setRootButton(target);
+            });
+
         new N2ToolbarButtonClick('#linear-solver-button', tooltipBox,
-            "Control Solver Tree Display",
+            "Control solver tree display",
             function (target) {
                 n2ui.setSolvers(true);
                 n2ui.showSolvers();
@@ -311,8 +332,8 @@ class N2Toolbar {
             "Save to SVG", e => { n2ui.n2Diag.saveSvg() });
 
         new N2ToolbarButtonToggle('#info-button', tooltipBox,
-            ["Show detailed node information", "Hide detailed node information"],
-            pred => { return n2ui.nodeInfoBox.hidden; },
+            ["Hide detailed node information", "Show detailed node information"],
+            pred => { return n2ui.nodeInfoBox.active; },
             e => {
                 n2ui.nodeInfoBox.clear();
                 n2ui.nodeInfoBox.toggle();
@@ -321,14 +342,8 @@ class N2Toolbar {
 
         new N2ToolbarButtonToggle('#question-button', tooltipBox,
             ["Hide N2 diagram help", "Show N2 diagram help"],
-            pred => { return d3.select("#myModal").style('display') == "block"; },
-            e => {
-                d3.select("#myModal").style('display', 'block');
-                d3.select("#idSpanModalClose").on('click', c => {
-                    d3.select("#myModal").style('display', 'none');
-                    d3.select("#idSpanModalClose").on('click', null);
-                });
-            }
+            pred => { return !!(d3.select(".window-theme-help").size()); },
+            e => { new N2Help() }
         );
 
         new N2ToolbarButtonToggle('#hide-toolbar', tooltipBox,
